/*
 * Copyright (c) 2004, 2005, 2006
 *      Luiz Souza <loos.br@gmail.com>.  All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <unistd.h>

#include "io.h"
#include "string.h"
#include "syslog.h"
#include "vchkuser.h"

int valid_user_char(char ch)
{
    /* from vpopmail - i have nothing with this */
    if (ch == 33 || ch == 35   ||
	ch == 36 || ch == 38   ||
	ch == 39 || ch == 42   ||
	ch == 43 || ch == 35   ||
	(ch >= 45 && ch <= 57) ||
	ch == 61 || ch == 63   ||
	(ch >= 65 && ch <= 90) ||
	(ch >= 94 && ch <= 126))
	return(OK);

    return(FAIL);
}

int read_user(string *user, char ch)
{
    if (valid_user_char(ch) == FAIL) {
	vlog("bad rcpt - invalid char for user");
	return(BOUNCE);
    }
    if (ch >= 65 && ch <= 90)
	ch += 32;
    if (str_cat(user, &ch, 1) == 0) {
	vlog("fail - no memory");
	return(FAIL);
    }
    return(OK);
}

int valid_domain_char(char ch)
{
    /* from vpopmail - i have nothing with this */
    if (ch == 45 || ch == 46   ||
	(ch >= 48 && ch <= 57) ||
	(ch >= 65 && ch <= 90) ||
	(ch >= 97 && ch <= 122))
	return(OK);

    return(FAIL);
}

int read_domain(string *domain, char ch)
{
    if (valid_domain_char(ch) == FAIL) {
	vlog("bad rcpt - invalid char for domain");
	return(BOUNCE);
    }
    if (ch >= 65 && ch <= 90)
	ch += 32;
    if (str_cat(domain, &ch, 1) == 0) {
	vlog("fail - no memory");
	return(FAIL);
    }
    return(OK);
}

int read_address(string *user, string *domain, char *env)
{
 char ch;
 int is_user = 1;

    /* check for null env - not set probably */
    if (env == NULL) {
	vlog("can't read e-mail");
	return(FAIL);
    }

    for(;;) {

	ch = *env++;

	/* check for eof */
	if (ch == 0)
	    break;

	/* check for a domain */
	if (ch == '@') {
	    is_user = 0;
	    continue;
	}

	if (is_user) {
	    switch (read_user(user, ch)) {
		case FAIL:
		    return(FAIL);
		case BOUNCE:
		    return(BOUNCE);
	    }
	} else {
	    switch (read_domain(domain, ch)) {
		case FAIL:
		    return(FAIL);
		case BOUNCE:
		    return(BOUNCE);
	    }
	}
    }
    return(NOTBOUNCE);
}
