/*
 * Copyright (c) 2004, 2005, 2006
 *      Luiz Souza <loos.br@gmail.com>.  All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>

#include "string.h"
#include "syslog.h"
#include "vchkuser.h"

int vbounce(string *domain_path)
{
 int		bounce = BOUNCE;
 string		dotqmail;
 char		buf[BUF];
 FILE		*fp;

    /* zero dotqmail */
    str_zero(&dotqmail);

    /* prepare .qmail-default file path */
    if (str_copy(&dotqmail, domain_path->s, domain_path->len) == 0) {
	vlog("fail - no memory");
	return(FAIL);
    }
    if (str_cat(&dotqmail, "/.qmail-default", 15) == 0) {
	vlog("fail - no memory");
	str_free(&dotqmail);
	return(FAIL);
    }

    /* open .qmail-default */
    if ((fp = fopen(dotqmail.s, "r")) == NULL) {
	vlog("could not open %s/.qmail-default", domain_path->s);
	return(FAIL);
    }

    /* read .qmail-default */
    while (fgets(buf, sizeof(buf), fp)) {

	/* skip comments and blank lines */
	if (*buf == '#' || *buf == '\n' || *buf == '\r')
	    continue;

	/* look for vdelivermail */
	if (strstr(buf, "vdelivermail")) {

	    /* check if the default action for vdelivermail is bounce */
	    if (strstr(buf, "bounce-no-mailbox") == NULL) {

		/* default action is not bounce */
		vlog("address bypass - default action for domain is not bounce");

		bounce = NOTBOUNCE;
	    }

	    /* 
	     * found vdelivermail, we are not interested in other instructions
	     */
	    break;
	}
    }
    (void)fclose(fp);
    str_free(&dotqmail);

    return(bounce);
}
