/*
 * Copyright (c) 2004, 2005, 2006
 *      Luiz Souza <loos.br@gmail.com>.  All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include "string.h"
#include "syslog.h"
#include "valias.h"
#include "vuser.h"
#include "vbounce.h"
#include "vdomain.h"
#include "vchkuser.h"
#include "read_address.h"


int main()
{
 string user;
 string domain;
 string domain_path;

    /* check for uid == vpopmail
     * exit on error
     * XXX - at this time we only _expect_ to run as vpopmail
     * no check is done
     */

    /* init strings - safe start */
    str_zero(&user);
    str_zero(&domain);
    str_zero(&domain_path);

    /*
     * these tests are mandatory, so any BOUNCE return should exit 
     * immediately
     *
     * functions fails could be catched other way, but as default
     * we just let the job to vpopmail (to not block valid address)
     *
     * if we see a NOTBOUNCE the address must continue
     */

    /* read e-mail from "REMOTERCPT" */
    switch (read_address(&user, &domain, getenv("REMOTERCPT"))) {
	case BOUNCE:
	    exit(BOUNCE);

	case FAIL:
	    exit(NOTBOUNCE);
    }
    // vlog("check for %s@%s", user.s, domain.s);
    // no need for instanciation of syslog, we will be one-liner logger

    /*
     * no domain - deny
     */
    if (domain.len == 0)
	exit(BOUNCE);

    /*
     * check if domain exist
     */
    switch (vdomain(&domain_path, &domain)) {
	case BOUNCE:
	    exit(BOUNCE);

	case FAIL:
	    exit(NOTBOUNCE);
    }

    /*
     * now we just exit if the function fail (see above) or if function
     * return NOTBOUNCE.
     *
     * if function return BOUNCE, the information isnt found by function
     * (but the next function may found - so let it decide)
     *
     * the last call should be BOUNCE by a direct exit call, so if any
     * of these functions trap before end, the address will be refused.
     */

    /*
     * this function checks if this domain return bounce messages
     * if not, exit here and let vpopmail deal with it
     */
    switch (vbounce(&domain_path)) {
	case FAIL:		/* fallthrought */
	case NOTBOUNCE:
	    /* do not proceed with tests, the default action is not bounce */
	    exit(NOTBOUNCE);
    }

    /*
     * if the user is a real vpopmail user, exit here and accept the
     * address
     */
    switch (vuser(&user, &domain)) {
	case FAIL:		/* fallthrought */
	case NOTBOUNCE:
	    exit(NOTBOUNCE);
    }

    /*
     * if address is an alias, exit here and accept the address
     */
    switch (valias(&user, &domain_path)) {
	case FAIL:		/* fallthrought */
	case NOTBOUNCE:
	    exit(NOTBOUNCE);
    }

    /* if no one has stopped us until this moment, goodbye address */
    vlog("bad rcpt - address refused, no more tests");
    exit(BOUNCE);
}
